;################################################################
;## Copyright (C)2003 Sony Computer Entertainment America Inc.

;---
;
; fluff.vcl
; Kirk Bender, SCEA
; draws lots of bouncy points
; entry in SCEA 2003 Developers Conference VU coding competition


; macro to load immediate

	.macro	loixyzw	reg,xval,yval,zval,wval
	loi		\xval
	addi.x		\reg,vf00,i

	loi		\yval
	addi.y		\reg,vf00,i

	loi		\zval
	addi.z		\reg,vf00,i

	loi		\wval
	muli.w		\reg,vf00,i
	.endm


kGifPacket .assign 0

.init_vf_all
.init_vi_all
.syntax new

.vu

--enter
--endenter

START:

; initialize registers
	iaddiu		idataindex, vi00, 3    		; load data index, data starts at QW #3
	iaddiu		ipacket,vi00, 0			; address of GIF packet
	iaddiu		ivertcount,vi00,1000 		; number of verts

	iaddi		iminus1, vi00, -1		; load iminus1 with -1
	mfir.xy		fminus1,iminus1                 ; move integer -1 to fminus1
	itof0.xy	fminus1,fminus1                 ; fminus1 = -1.0

	lqi.xyz		foffset,  (idataindex++)	; get coord offset
	lqi.xyz		fscale,	 (idataindex++)		; get scale
	lqi.zw		fgrav, (idataindex++)		; get gravity
	lqi.xy		fdamp, (idataindex++)		; get damp

	;iaddiu		iinitted,vi00,0			; to avoid used before initialized errors...
	ilwr.x		iinitted, (idataindex)		; get initted flag
	iaddiu		idataindex, idataindex, 1
	ibgtz  		iinitted, eachvert		; if already initted, skip initialization

;---------------------------------------------
; init verts

	iaddiu		itemp,vi00, 2			; random scale
	mfir.xyzw	frandscale,itemp
	itof0.xyzw	frandscale,frandscale

	iaddiu		itemp,vi00, 1			; random offset
	mfir.xyzw	frandoffs,itemp
	itof0.xyzw	frandoffs,frandoffs

	iaddiu		itemp,vi00, 10   		; position range
	mfir.xy		frandrange,itemp
	itof0.xy	frandrange,frandrange

	iaddiu		itemp,vi00, 4			; dx,dy range
	mfir.zw		fdrandrange,itemp
	itof0.zw	fdrandrange,fdrandrange


	; random seed init values
	loixyzw		rSeed,1.31357231,1.79512412,17.2341241,3.214341
	loixyzw		rMul,1.31357231,1.79512412,17.2341241,3.214341
	loixyzw		rAdd,0.71234121,0.1245121,0.313815147,0.6185812

initeachvert:     					; get random numbers for all verts

	; Stewart's random number generator, gave up on the built-in one
	mula		rSeed,rSeed,rMul   
	maddw		rSeed,rAdd,vf00
	ftoi0		rI,rSeed
	itof0		rI,rI
	sub		rSeed,rSeed,rI			; rSeed now [0-1.0)

	; convert 0-1.0 to the kind of numbers we want
	mul.xyzw	ftemp,rSeed,frandscale
	sub.xyzw	ftemp,ftemp,frandoffs
	mul.xy		ftemp,ftemp,frandrange
	mul.zw		ftemp,ftemp,fdrandrange


	sqi.xyzw    	ftemp,(idataindex++)		; store the vert

	; all verts?
	isubiu		ivertcount,ivertcount,1      	; vertcount-=1
	ibne		ivertcount,vi00, initeachvert	; if vertcount!=0 goto initeachvert

	; done initting verts

	iaddiu		ivertcount,vi00,1000 		; reset number of verts

	iaddiu 		idataindex,vi00,7     		; reset data index
	iaddiu		itemp2,vi00,1

	iswr.x    	itemp2,(idataindex)		; set initted flag=1, don't init verts next time
 	iaddiu		idataindex, idataindex, 1


;------------------------------------------------

; process vertices
eachvert: ; do for each vert
	lq.xyzw         fvector, (idataindex) 		; get vector x,y,dx,dy
	move.xyzw	fcoord,vf00			; zero out fcoord
	move.xy		fcoord,fvector			; get x,y

	; extract dx, dy from vert data:  rotate xy <- zw
	mr32.xyzw	fvector,fvector                 ; rotate
	mr32.xyzw	fvector,fvector                 ; rotate
	add.xy		fcoord,fcoord,fvector		; add dx,dy to x,y

; bounds check
; check x

	fcset 		0  				; reset clipping stuff
	iaddiu		ibound,vi00,150			; load x bound value, hardcoded-- sorry.
	mfir.w		fbound,ibound
	itof0.w		fbound,fbound
	clipw.xyz	fcoord,fbound
	fcand		vi01,0x1 			; check +x
	ibgtz		vi01, bouncex
	b checknegx
bouncex:  						; hit an x boundary
	mul.x 		fvector,fvector,fminus1		; reverse direction:  dx=  (-1.0)*dx
	mul.xy 		fvector,fvector,fdamp		; slow it down: dx= (damping factor)*dx

	b checky
checknegx:
	fcand		vi01,0x2 			; check -x
	ibgtz		vi01, bouncex

; check y

checky:
	fcset 		0  				; reset clipping stuff
	iaddiu		ibound,vi00,75			; load y bound value, hardcoded-- sorry.
	mfir.w		fbound,ibound
	itof0.w		fbound,fbound
	clipw.xyz	fcoord,fbound

	fcand		vi01, 0x04 			; check +y
	ibgtz		vi01, bouncey
	b checknegy

bouncey:  						; hit a y boundary
	mul.y 		fvector,fvector,fminus1   	; reverse direction: dy=  (-1.0)*dy
	mul.xy 		fvector,fvector,fdamp		; slow it down: dy= (damping factor)*dy
	b render
checknegy:
	fcand		vi01, 0x08 			; check -y
	ibgtz		vi01, bouncey

render:
	sq.xy		fcoord, (idataindex)	 	; store x, y back in vu mem, don't increment


	; rotate zw <- xy
	mr32.xyzw	fvector,fvector                 ; rotate
	mr32.xyzw	fvector,fvector                 ; rotate
	add.zw		fvector,fvector,fgrav		; add gravity
	sqi.zw		fvector, (idataindex++)	 	; store dx, dy back in vu mem

	mul.xyz		fcoord,fcoord,fscale		; multiply by scale factor
	add.xyz		fcoord,fcoord,foffset    	; add screen offset
	ftoi4.xyz	fcoord,fcoord			; convert vert to 12:4 format for GS
	iaddiu		ipacketcoord,ipacket,2 		; make address of coords in packet
	sq.xyz		fcoord,(ipacketcoord)  		; store coord in packet

	; Fortunately point primitives are fast enough, but if they were triangles
	; we'd have to do double buffering of the primitives, otherwise a packet
	; might be overwritten before it has a chance to be is sent.
	; so we just do single buffering, kick, and forget about it.

	xgkick		ipacket     			; kick packet

;end of verts?

	isubiu		ivertcount,ivertcount,1		; vertcount-=1
	ibne		ivertcount,vi00, eachvert	; if vertcount!=0 goto eachvert

; that's all she wrote

--exit
--endexit






