//-----------------------------------------------------------
// PS2 VU/VIF related functionality
// Low-level VIF/VU functions
//
// Geoff Audy, November 8th 2001
//
// Copyright (C)2002 Sony Computer Entertainment America Inc.
//
// This file is subject to the terms and conditions of the GNU
// General Public Licence Version 2.  See the file "COPYING"
// in the main directory of this archive for more details.
//-----------------------------------------------------------

#ifndef __PS2_VIF_H__
#define __PS2_VIF_H__

#ifdef __cplusplus
extern "C" {
#endif

//-------------------------------------
enum
{
   VIF_RET_OK = 0,
   VIF_RET_ALREADYINITIALIZED,
   VIF_RET_NOTINITIALIZED,
   VIF_RET_INVALIDPARAMETER,
   VIF_RET_WRONGALIGNMENT,
   VIF_RET_INCOMPLETE,
   VIF_RET_COMPLETE,
   VIF_RET_DATATOOBIGTOFIT,
   VIF_RET_ERROR,

   VIF_RET_MAX
};

enum
{
   VIF_DEST_VIF0 = 0,
   VIF_DEST_VIF1,

   VIF_DEST_MAX
};

enum
{
   VIF_WAIT_NONE = 0,
   VIF_WAIT_VIF0,
   VIF_WAIT_VIF1,
   VIF_WAIT_VIF0ANDVIF1,

   VIF_WAIT_MAX
};

enum
{
   VIF_BLOCKING_BLOCKING = 0,
   VIF_BLOCKING_NONBLOCKING,

   VIF_BLOCKING_MAX
};

enum
{
   VIF_ATTACHMENT_INLINE = 0,
   VIF_ATTACHMENT_REFERENCE,
   VIF_ATTACHMENT_CALL,

   VIF_ATTACHMENT_MAX
};

enum
{
   VIF_UPLOAD_IFNECESSARY = 0,
   VIF_UPLOAD_FORCE,

   VIF_UPLOAD_MAX
};

#define  VIF_CMD_NOP       0x00
#define  VIF_CMD_STCYCL    0x01
#define  VIF_CMD_OFFSET    0x02
#define  VIF_CMD_BASE      0x03
#define  VIF_CMD_ITOP      0x04
#define  VIF_CMD_STMOD     0x05
#define  VIF_CMD_MSKPATH3  0x06
#define  VIF_CMD_MARK      0x07
#define  VIF_CMD_FLUSHE    0x10
#define  VIF_CMD_FLUSH     0x11
#define  VIF_CMD_FLUSHA    0x13
#define  VIF_CMD_MSCAL     0x14
#define  VIF_CMD_MSCNT     0x17
#define  VIF_CMD_MSCALF    0x15
#define  VIF_CMD_STMASK    0x20
#define  VIF_CMD_STROW     0x30
#define  VIF_CMD_STCOL     0x31
#define  VIF_CMD_MPG       0x4A
#define  VIF_CMD_DIRECT    0x50
#define  VIF_CMD_DIRECTHL  0x51
#define  VIF_CMD_UNPACK    0x60

#define  VIF_CMD_MAX       0x70

// Use this in the flag field of unpack viftag to
// specify unpack relative to the current buffer
#define  VIF_TOPS_REL      0x8000

#define  VIF_UNPACK_S_32   0x0
#define  VIF_UNPACK_S_16   0x1
#define  VIF_UNPACK_S_8    0x2
#define  VIF_UNPACK_V2_32  0x4
#define  VIF_UNPACK_V2_16  0x5
#define  VIF_UNPACK_V2_8   0x6
#define  VIF_UNPACK_V3_32  0x8
#define  VIF_UNPACK_V3_16  0x9
#define  VIF_UNPACK_V3_8   0xA
#define  VIF_UNPACK_V4_32  0xC
#define  VIF_UNPACK_V4_16  0xD
#define  VIF_UNPACK_V4_8   0xE
#define  VIF_UNPACK_V4_5   0xF

//-------------------------------------
unsigned int VIF_ModuleInit (OS_Qword *doublebuffervif0,
                             unsigned int doublebuffervif0size,
                             OS_Qword *doublebuffervif1,
                             unsigned int doublebuffervif1size);
unsigned int VIF_ModuleUninit (void);

unsigned int VIF_AddTag (unsigned int command, unsigned int num,
                         unsigned int immediate, unsigned int dest);
unsigned int VIF_Add4Tags (unsigned int command1, unsigned int num1,
                           unsigned int immediate1,
                           unsigned int command2, unsigned int num2,
                           unsigned int immediate2,
                           unsigned int command3, unsigned int num3,
                           unsigned int immediate3,
                           unsigned int command4, unsigned int num4,
                           unsigned int immediate4,
                           unsigned int dest);
unsigned int VIF_AddCode (OS_Qword *code, unsigned int dest, unsigned int forceupload);
unsigned int VIF_AddVUData (OS_Qword *buffer, unsigned int buffersizeinqw,
                            unsigned int destvuaddr, unsigned int attachment,
                            unsigned int dest);
unsigned int VIF_AddData (OS_Qword *buffer, unsigned int buffersizeinqw,
                          unsigned int attachment, unsigned int dest);
unsigned int VIF_AddTextureAndClut (OS_Qword *texture, OS_Qword *clut,
                                    unsigned int width, unsigned int height,
                                    unsigned int bitdepth, unsigned int vramaddresstexture,
                                    unsigned int vramaddressclut);

unsigned int VIF_SwapBuffers (unsigned int dest);

unsigned int VIF_TransferWait (unsigned int dest);

unsigned int VIF_GetChainSize (unsigned int dest, unsigned int *chainsize);

#ifdef __cplusplus
}
#endif

#endif   // __PS2_VIF_H__
